@@ -10,6 +10,8 @@ our @ISA = qw/Exporter/;
 our @EXPORT_OK = qw( scripts_in_code);
 our @EXPORT = @EXPORT_OK;
 
+our $VERSION="2.0.99";
+
 our @MANIFEST = (
 'clear.sh',
 'my.sandbox.cnf',
@@ -23,31 +25,25 @@ our @MANIFEST = (
 'my.sh',
 'change_paths.sh',
 'change_ports.sh',
-#'COPYING',
-#'README',
 'USING',
 'grants.mysql',
-# 'current_options.conf',
 '# INSTALL FILES',
-# 'sandbox',
-##'make_sandbox',
-##'low_level_make_sandbox',
-##'make_replication_sandbox',
-##'make_multiple_sandbox',
-##'make_multiple_custom_sandbox',
-##'sbtool',
 'sandbox_action.pl',
-#'tests/test_sandbox.pl',
-#'tests/README',
-#'sandbox.conf',
-##'Changelog',
-
 );
 
 sub manifest {
     return @MANIFEST;
 }
 my %parse_options_low_level_make_sandbox = (
+##
+# DEPRECATED and removed 
+# (April fool's edition)
+#    query_analyzer         => {
+#                                value => 0,
+#                                parse=> 'query_analyzer',
+#                                so  => 5,
+#                                help => ['solves your tuning problems at once']
+#    },
     upper_directory        => {
                                 value => $ENV{'SANDBOX_HOME'} || $ENV{'HOME'},       
                                 parse => 'upper_directory=s',         
@@ -62,7 +58,7 @@ my %parse_options_low_level_make_sandbox = (
                                 parse => 'd|sandbox_directory=s',      
                                 so    =>  20,
                                 help  => [
-                                            'Where to install the sandbox, under home-directory'
+                                            'Where to install the sandbox, under upper-directory'
                                          ] 
                              },
     sandbox_port          => {
@@ -74,6 +70,26 @@ my %parse_options_low_level_make_sandbox = (
                                             '(Default: 3310)',
                                          ]
                             },
+    check_port            => {
+                                value => 0,               
+                                parse => 'check_port',           
+                                so    =>  35,
+                                help  => [
+                                            'Check whether the provided port is free,', 
+                                            'and determine the next available one if it is not.',
+                                            '(Default: disabled)',
+                                         ]
+                            },
+    no_check_port         => {
+                                value => 0,               
+                                parse => 'no_check_port',           
+                                so    =>  36,
+                                help  => [
+                                            'Ignores requests of checking ports.', 
+                                            'To be used by group sandbox scripts.',
+                                            '(Default: disabled)',
+                                         ]
+                            },
     datadir_from          => {
                                 value => 'script',          
                                 parse => 'datadir_from=s' ,            
@@ -231,8 +247,14 @@ my %parse_options_low_level_make_sandbox = (
                                             'Use this option to be guided through the installation process (default: disabled)'
                                          ] 
                             },
-    more_options          => {value => q{},                 parse => undef,                        so => 20},
-    help                  => {value => q{},                 parse => 'help',                       so => 25},
+    more_options          => {
+                                value => q{},                 
+                                parse => undef,                        
+                                so => 20},
+    help                  => {
+                                value => q{},                 
+                                parse => 'help',                       
+                                so => 25},
     no_confirm            => {
                                 value => 0, 
                                 parse => 'no_confirm',
@@ -318,6 +340,15 @@ my %parse_options_replication = (
                                          ] 
                             },
  
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -399,6 +430,16 @@ my %parse_options_many = (
                                             'set the nodes in circular replication'
                                          ] 
                             },
+
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -454,6 +495,15 @@ my %parse_options_custom_many = (
                                          ]
                             },
 
+    interactive           => {
+                                value => 0,                  
+                                parse => 'interactive',                  
+                                so    => 65,
+                                help  => [
+                                            'Use this option to ask interactive user ',
+                                            'confirmation for each node (default: disabled)'
+                                         ] 
+                            },
     verbose               => {
                                 value => 0,                  
                                 parse => 'v|verbose',                  
@@ -472,6 +522,140 @@ my %parse_options_custom_many = (
                             },
 ); 
 
+our %sbtool_supported_operations = (
+    ports => 'lists ports used by the Sandbox',
+    range => 'finds N consecutive ports not yet used by the Sandbox',
+    info  => 'returns configuration options from a Sandbox',
+    tree  => 'creates a replication tree',
+    copy  => 'copies data from one Sandbox to another',
+    move  => 'moves a Sandbox to a different location',
+    port  => 'Changes a Sandbox port',
+    delete => 'removes a sandbox completely',
+    preserve => 'makes a sandbox permanent',
+    unpreserve => 'makes a sandbox NOT permanent',
+);
+
+our %sbtool_supported_formats = (
+    text => 'plain text dump of requested information',
+    perl => 'fully structured information in Perl code',
+);
+
+
+my %parse_options_sbtool = (
+    operation => {
+        so       => 10,
+        parse    => 'o|operation=s',
+        value    => undef,
+        accepted => \%sbtool_supported_operations,
+        help     => 'what task to perform',
+    },
+    source_dir => {
+        so    => 20,
+        parse => 's|source_dir=s',
+        value => undef,
+        help  => 'source directory for move, copy, delete',
+    },
+    dest_dir => {
+        so    => 30,
+        parse => 'd|dest_dir=s',
+        value => undef,
+        help  => 'destination directory for move,copy',
+    },
+    new_port  => {
+        so    => 40,
+        parse => 'n|new_port=s',
+        value => undef,
+        help  => 'new port while moving a sandbox',
+    },
+    only_used => {
+        so    => 50,
+        parse => 'u|only_used',
+        value => 0,
+        help  => 'for "ports" operation, shows only the used ones',
+    },
+    min_range => {
+        so    => 60,
+        parse => 'i|min_range=i',
+        value => 5000,
+        help  => 'minimum port when searching for available ranges',
+    },
+    max_range => {
+        so    => 70,
+        parse => 'x|max_range=i',
+        value => 64000,
+        help  => 'maximum port when searching for available ranges',
+    },
+    range_size => {
+        so    => 80,
+        parse => 'z|range_size=i',
+        value => 10,
+        help  => 'size of range when searching for available port range',
+    },
+    format => {
+        so       => 90,
+        parse    => 'f|format=s',
+        value    => 'text',
+        accepted => \%sbtool_supported_formats,
+        help     => 'format for "ports" and "info"',
+    },
+    search_path => {
+        so    => 100,
+        parse => 'p|search_path=s',
+        value => $ENV{SANDBOX_HOME},
+        help  => 'search path for ports and info',
+    },
+    all_info => {
+        so    => 110,
+        parse => 'a|all_info',
+        value => 0,
+        help  => 'print more info for "ports" operation'
+    },
+    master_node => {
+        so    => 115,
+        parse => 'master_node=i',
+        value => '1',
+        help  => 'which node should be master (default: 1)',
+    },
+    tree_nodes => {
+        so    => 120,
+        parse => 'tree_nodes=s',
+        value => '',
+        help  => 'description of the tree (x-x x x-x x|x x x|x x)',
+    },
+    mid_nodes => {
+        so    => 130,
+        parse => 'mid_nodes=s',
+        value => '',
+        help  => 'description of the middle nodes (x x x)',
+    },
+    leaf_nodes => {
+        so    => 140,
+        parse => 'leaf_nodes=s',
+        value => '',
+        help  => 'description of the leaf nodes (x x|x x x|x x)',
+    },
+    tree_dir => {
+        so    => 150,
+        parse => 'tree_dir=s',
+        value => '',
+        help  => 'which directory contains the tree nodes',
+    },
+    verbose => {
+        so    => 160,
+        parse => 'v|verbose',
+        value => 0,
+        help  => 'prints more info on some operations'
+    },
+    help => {
+        so    => 999,
+        parse => 'h|help',
+        value => undef,
+        help  => 'this screen',
+    },
+);
+
+
+
 my %scripts_in_code = (
     'start.sh' => <<'START_SCRIPT',
 #!_BINBASH_
@@ -489,7 +673,7 @@ then
 else
     CURDIR=`pwd`
     cd $BASEDIR
-    if [ "$DEBUG" = "" ]
+    if [ "$SBDEBUG" = "" ]
     then
         $MYSQLD_SAFE --defaults-file=$SBDIR/my.sandbox.cnf $@ > /dev/null 2>&1 &
     else
@@ -763,7 +947,7 @@ PERL_SCRIPT3='s/\b$old\b/$new/'
 PERL_SCRIPT="$PERL_SCRIPT1 $PERL_SCRIPT2 $PERL_SCRIPT3"
 
 SCRIPTS1="start stop send_kill clear restart my.sandbox.cnf "
-SCRIPTS2="load_grants my use current_options.conf $0"
+SCRIPTS2="load_grants my use $0"
 SCRIPTS="$SCRIPTS1 $SCRIPTS2"
 for SCRIPT in $SCRIPTS
 do
@@ -804,11 +988,11 @@ then
 fi
 
 PERL_SCRIPT1='BEGIN{$old=shift;$new=shift};'
-PERL_SCRIPT2='s/\b$old\b/$new/g' 
+PERL_SCRIPT2='s/$old/$new/g' 
 PERL_SCRIPT="$PERL_SCRIPT1 $PERL_SCRIPT2"
 
 SCRIPTS1="start stop send_kill clear restart my.sandbox.cnf "
-SCRIPTS2="load_grants my use current_options.conf $0"
+SCRIPTS2="load_grants my use $0"
 SCRIPTS="$SCRIPTS1 $SCRIPTS2"
 
 for SCRIPT in $SCRIPTS
@@ -869,7 +1053,7 @@ SANDBOX_ACTION_SCRIPT
 
 my $license_text = <<'LICENSE';
 #    The MySQL Sandbox
-#    Copyright (C) 2006,2007,2008 Giuseppe Maxia, MySQL AB
+#    Copyright (C) 2006,2007,2008,2009 Giuseppe Maxia
 #    Contacts: http://datacharmer.org
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -902,6 +1086,10 @@ sub parse_options_many {
     return \%parse_options_many;
 }
 
+sub parse_options_sbtool {
+    return \%parse_options_sbtool;
+}
+
 sub parse_options_custom_many {
     return \%parse_options_custom_many;
 }
